(* Tool for GnuGetText for Delphi
   Insert the MsgId from one po file as MsgStr into another
   use "#. Programmer''s name for it:" as criteria

   !!! experimental version

    2015 Dr. J. Rathlev, D-24222 Schwentinental (kontakt(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Feb. 2014
   last modified: February 2023
   *)

unit PoInsMain;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes,
  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.Buttons,
  Vcl.ExtCtrls, PoParser;

const
  Vers = ' - Vers. 3.1';
  CopRgt = ' 2014-2024 Dr. J. Rathlev, 24222 Schwentinental';
  EmailAdr = 'kontakt(a)rathlev-home.de';

  PoExt = 'po';

type
  TfrmMain = class(TForm)
    pnMain: TPanel;
    bbExit: TBitBtn;
    SaveDialog: TSaveDialog;
    bbInfo: TBitBtn;
    pnTools: TPanel;
    laEntry: TLabel;
    bbSave: TBitBtn;
    sbEdit: TSpeedButton;
    sbTrans: TSpeedButton;
    Label2: TLabel;
    Label5: TLabel;
    OpenDialog: TOpenDialog;
    cbEdit: TComboBox;
    cbTrans: TComboBox;
    procedure FormCreate(Sender: TObject);
    procedure bbExitClick(Sender: TObject);
    procedure bbInfoClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure cbTransCloseUp(Sender: TObject);
    procedure cbEditCloseUp(Sender: TObject);
    procedure bbSaveClick(Sender: TObject);
    procedure sbEditClick(Sender: TObject);
    procedure sbTransClick(Sender: TObject);
  private
    { Private-Deklarationen }
    ProgVersName,
    ProgVersDate,
    AppPath,UserPath,
    IniName,ProgPath,
    EdFile,CompFile     : string;
    EdList,CompList     : TPoEntryList;
    PeRef              : TPoEntry;
    IdList             : TStringList;
    Loaded,Changed     : boolean;
    IdIndex            : integer;
    procedure SaveFile;
    function LoadFiles (LoadRef : boolean) : boolean;
    function SelectComp : boolean;
    function SelectEdit : boolean;
  public
    { Public-Deklarationen }
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.dfm}

uses System.IniFiles, Winapi.ShlObj, System.StrUtils, WinUtils, ListUtils, LangUtils,
  InitProg, WinApiUtils, WinShell, gnugettext, PathUtils, MsgDialogs, GgtConsts;

{ ------------------------------------------------------------------- }
procedure TfrmMain.FormCreate(Sender: TObject);
var
  IniFile  : TMemIniFile;
begin
  TranslateComponent (self);
  Application.Title:=_('Insert translations');
  InitPaths(AppPath,UserPath,ProgPath);
  InitVersion(Application.Title,Vers,CopRgt,3,3,ProgVersName,ProgVersDate);
  Caption:=ProgVersName;
  IniName:=Erweiter(AppPath,PrgName,IniExt);
  IniFile:=TMemIniFile.Create(IniName);
  with IniFile do begin
    EdFile:=ReadString(CfGSekt,iniTempl,'');
    CompFile:=ReadString(CfGSekt,iniTrans,'');
    LoadHistory(IniFile,TemplSekt,cbEdit);
    LoadHistory(IniFile,TransSekt,cbTrans);
    Free;
    end;
  AddToHistory(cbEdit,EdFile);
  AddToHistory(cbTrans,CompFile);
  EdList:=TPoEntryList.Create;
  CompList:=TPoEntryList.Create;
  IdList:=TStringList.Create;
  Loaded:=false; Changed:=false;
  end;

procedure TfrmMain.FormClose(Sender: TObject; var Action: TCloseAction);
var
  IniFile  : TMemIniFile;
begin
  IniFile:=TMemIniFile.Create(IniName);
  with IniFile do begin
    WriteString(CfGSekt,iniTempl,EdFile);
    WriteString(CfGSekt,iniTrans,CompFile);
    SaveHistory(IniFile,TemplSekt,cbEdit);
    SaveHistory(IniFile,TransSekt,cbTrans);
    UpdateFile;
    Free;
    end;
  IdList.Free;
  EdList.Free;
  CompList.Free;
  end;

procedure TfrmMain.bbExitClick(Sender: TObject);
begin
  Close;
  end;

procedure TfrmMain.bbInfoClick(Sender: TObject);
begin
  InfoDialog(BottomLeftPos(bbInfo,0,10),ProgVersName+' - '+ProgVersDate+#13+CopRgt
           +#13'E-Mail: '+EmailAdr);
  end;

function TfrmMain.SelectComp : boolean;
begin
  with OpenDialog do begin
    if length(CompFile)>0 then InitialDir:=ExtractFilePath(CompFile)
    else InitialDir:=UserPath;
    Filename:='';
    Title:=_('Select po file used for comparison');
    Filter:=Format(_('po files|*.%s|all|*.*'),[PoExt]);
    if Execute then begin
      CompFile:=Filename;
      AddToHistory(cbTrans,Filename);
      Result:=true;
      end
    else Result:=false;
    end;
  end;

function TfrmMain.SelectEdit : boolean;
begin
  with OpenDialog do begin
    if length(EdFile)>0 then InitialDir:=ExtractFilePath(EdFile)
    else InitialDir:=UserPath;
    Filename:='';
    Title:=_('Select po file to be edited');
    Filter:=Format(_('po files|*.%s|all|*.*'),[PoExt]);
    if Execute then begin
      EdFile:=FileName;
      AddToHistory(cbEdit,Filename);
      Result:=true;
      end
    else Result:=false;
    end;
  end;

procedure TfrmMain.sbTransClick(Sender: TObject);
begin
  SelectComp;
  end;

procedure TfrmMain.sbEditClick(Sender: TObject);
begin
  SelectEdit;
  end;

procedure TfrmMain.cbTransCloseUp(Sender: TObject);
begin
  with cbTrans do begin
    CompFile:=Items[ItemIndex];
    end;
  end;

procedure TfrmMain.cbEditCloseUp(Sender: TObject);
begin
  with cbEdit do begin
    EdFile:=Items[ItemIndex];
    end;
  end;

function TfrmMain.LoadFiles (LoadRef : boolean) : boolean;
var
  pr,pe : TPoEntry;
  si,s : string;
  tf : TextFile;
  ok : boolean;
  ne : integer;

  function ProgrName (po : TPoEntry) : string;
  var
    i : integer;
  const
    ps = '#. Programmer''s name for it: ';
  begin
    with po.AutoCommentList do begin
      for i:=0 to Count-1 do if AnsiStartsText(ps,Strings[i]) then Break;
      if i<Count then begin
        Result:=Strings[i];
        System.Delete(Result,1,length(ps));
        end
      else Result:='';
      end;
    end;

  function FindProgrName (pl : TPoEntryList; const spn : string) : TPoEntry;
  var
    i,j : integer;
    pe  : TPoEntry;
    s   : string;
  begin
    pe:=pl.FindFirst; Result:=nil;
    while pe<>nil do begin
      if AnsiSameText(ProgrName(pe),spn) then begin
        Result:=pe; Break;
        end;
      pe:=pl.FindNext(pe);
      end;
    end;

begin
  IdList.Clear;
  Result:=false;
  if LoadRef then begin
    if not FileExists(EdFile) then begin
      laEntry.Caption:=Format(_('File not found: %s'),[EdFile]);
      Exit;
      end;
    Changed:=false;
    end;
  try
    if LoadRef then begin
      ne:=EdList.LoadFromFile(EdFile);
      if ne>0 then begin
        laEntry.Caption:=Format(_('Error in line %u of edit file!'),[ne]);
        Exit;
        end;
      end;
    ne:=CompList.LoadFromFile(CompFile);
    if ne>0 then begin
      laEntry.Caption:=Format(_('Error in line %u of comparison file!'),[ne]);
      Exit;
      end;
    with EdList do begin
      pr:=FindFirst;
      while (pr<>nil) do begin
        if (pr.MsgId<>'') and (copy(pr.MsgId,1,2)<>HistMarker) then begin
          si:=ProgrName(pr);
          pe:=FindProgrName(CompList,si);
          if pe<>nil then begin
            pr.MsgStr:=pe.MsgId;
            end;
          end;
        pr:=FindNext(pr);
        end;
      end;
    Result:=true;
  except
    laEntry.Caption:=_('Error reading po files!');
    end;
  end;

procedure TfrmMain.bbSaveClick(Sender: TObject);
begin
  if LoadFiles(true) then SaveFile;
  end;

procedure TfrmMain.SaveFile;
var
  sn,s   : string;
  ok     : boolean;
  i      : integer;
begin
  sn:=EdFile+'.tmp';
  EdList.SaveToFile(sn,true);
  s:=EdFile+'.bak';
  if FileExists(s) then DeleteFile(s);
//  RenameFile(EdFile,s);
//  RenameFile(sn,EdFile);
  bbSave.Enabled:=false; Changed:=false;
  end;

end.
