(* Tool for GnuGetText for Delphi
   translate po file - *** incomplete ***

    Dr. J. Rathlev, D-24222 Schwentinental (kontakt(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Note: The routines ExtractStrings, ExtractFromPascal and ExtractFromDFM
   originate from xgettexttools:
      (C) Copyright by Lars B. Dybdahl and Jens Berke
      E-mail: Lars@dybdahl.dk, phone +45 70201241
      You may distribute and modify this file as you wish for free

   Feb. 2021
   last modified: April 2024
   *)

unit PoTransMain;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.Buttons, Vcl.ComCtrls,
  PoParser, GgtConsts;

const
  Vers = ' - Vers. 3.1';

type
  TfrmMain = class(TForm)
    Label2: TLabel;
    sbOpenPoFile: TSpeedButton;
    bbInfo: TBitBtn;
    bbExit: TBitBtn;
    lvPoEntries: TListView;
    OpenDialog: TOpenDialog;
    bbTranslate: TBitBtn;
    bbApply: TBitBtn;
    edTranslation: TEdit;
    edPoFile: TComboBox;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormShow(Sender: TObject);
    procedure bbExitClick(Sender: TObject);
    procedure sbOpenPoFileClick(Sender: TObject);
    procedure lvPoEntriesDrawItem(Sender: TCustomListView; Item: TListItem;
      Rect: TRect; State: TOwnerDrawState);
    procedure FormResize(Sender: TObject);
    procedure bbTranslateClick(Sender: TObject);
  private
    { Private-Deklarationen }
    ProgVersName,
    ProgVersDate,
    AppPath,UserPath,
    IniName,ProgPath,
    PoFile             : string;
    PoList             : TPoEntryList;
    procedure ClearList;
    procedure ShowList;
    function LoadFile : boolean;
    function SelectPo: boolean;
  public
    { Public-Deklarationen }
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.dfm}

uses System.IniFiles, System.StrUtils, System.UITypes,
  GnuGetText, WinUtils, ListUtils, InitProg, PathUtils, MsgDialogs;

{ ------------------------------------------------------------------- }
const
  (* INI-Variables *)
  iniPoName = 'LastPo';

procedure TfrmMain.FormCreate(Sender: TObject);
var
  IniFile  : TMemIniFile;
begin
  TranslateComponent(self);
  Application.Title:=_('Translate PO file');
  InitPaths(AppPath,UserPath,ProgPath);
  InitVersion(Application.Title,Vers,CopRgt,3,3,ProgVersName,ProgVersDate);
  Caption:=ProgVersName;
  IniName:=Erweiter(AppPath,PrgName,IniExt);
  IniFile:=TMemIniFile.Create(IniName);
  with IniFile do begin
    PoFile:=ReadString(CfGSekt,iniPoName,'');
    LoadHistory(IniFile,TransSekt,edPoFile);
    Free;
    end;
  AddToHistory(edPoFile,PoFile);
  PoList:=TPoEntryList.Create;
  end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  PoList.Free;
  end;

procedure TfrmMain.FormResize(Sender: TObject);
var
  w : integer;
begin
  with lvPoEntries do begin
    w:=ClientRect.Right div 2;
    Columns[0].Width:=w; Columns[1].Width:=w;
    end;
  end;

procedure TfrmMain.FormClose(Sender: TObject; var Action: TCloseAction);
var
  IniFile  : TMemIniFile;
begin
  IniFile:=TMemIniFile.Create(IniName);
  with IniFile do begin
    WriteString(CfGSekt,iniPoName,PoFile);
    SaveHistory(IniFile,TransSekt,edPoFile);
    UpdateFile;
    Free;
    end;
  end;

procedure TfrmMain.FormShow(Sender: TObject);
var
  ok : boolean;
begin
  if FileExists(PoFile) then LoadFile
  else begin
    ok:=SelectPo;
    if ok then LoadFile
    end;
  end;

procedure TfrmMain.bbExitClick(Sender: TObject);
begin
  Close;
  end;

procedure TfrmMain.bbTranslateClick(Sender: TObject);
var
  n : integer;
begin
  n:=integer(lvPoEntries.Selected.Data);
  if n>=0 then with PoList[n] do begin

    end;
  end;

procedure TfrmMain.ClearList;
begin
  lvPoEntries.Clear;
  end;

procedure TfrmMain.ShowList;
var
  i : integer;
  s : string;
  pe : TPoEntry;
begin
  with PoList do begin
    pe:=FindFirst;
    while pe<>nil do begin
      with lvPoEntries.Items.Add do begin
        Caption:=pe.MsgId;
        SubItems.Add(pe.MsgStr);
        if pe.Fuzzy then s:='?' else s:='';
        SubItems.Add(s);
        Data:=pointer(i);
        end;
      pe:=FindNext(pe);
      end;
    end;
  lvPoEntries.ItemIndex:=0;
  end;

function TfrmMain.LoadFile : boolean;
var
  ne : integer;
begin
  Result:=false;
  if not FileExists(PoFile) then begin
    ErrorDialog(Format(_('File not found: %s'),[PoFile]));
    Exit;
    end;
  ClearList;
  try
    ne:=PoList.LoadFromFile(PoFile);
    if ne>0 then
      ErrorDialog(Format(_('Error in line %u of po file!'),[ne]))
    else ShowList;
  except
    ErrorDialog(_('Error reading po file!'));
    end;
  end;

function TfrmMain.SelectPo : boolean;
begin
  with OpenDialog do begin
    if length(PoFile)>0 then InitialDir:=ExtractFilePath(PoFile)
    else InitialDir:=UserPath;
    Filename:='';
    Title:=_('Select po file');
    Filter:=Format(_('po files|*.%s|all|*.*'),[PoExt]);
    if Execute then begin
      PoFile:=Filename;
      AddToHistory(edPoFile,PoFile);
      Result:=true;
      end
    else Result:=false;
    end;
  end;

procedure TfrmMain.lvPoEntriesDrawItem(Sender: TCustomListView; Item: TListItem;
  Rect: TRect; State: TOwnerDrawState);
var
  x : integer;
begin
  with lvPoEntries,Canvas do begin
    if length(Item.SubItems[1])=0 then begin
      if odSelected in State then Brush.Color:=clGradientActiveCaption
      else Brush.Color:=clWhite;
      end
    else begin
      if odSelected in State then Brush.Color:=TColorRec.Lightpink
      else Brush.Color:=TColorRec.Lemonchiffon;
      end;
    FillRect(Rect);
    Font.Color:=clCaptionText;
    x:=Rect.Left;
    TextOut(x,Rect.Top+1,Item.Caption);
    x:=x+Columns[0].Width;
    Font.Color:=TColorRec.Navy; //Royalblue;
    TextOut(x,Rect.Top+1,Item.SubItems[0]);
    end;
  end;

procedure TfrmMain.sbOpenPoFileClick(Sender: TObject);
begin
  if SelectPo then LoadFile;
  end;

end.
